
// 定义自定义覆盖物的构造函数
function MyMapOverlay(map, point, param) {
  this.map = map;
  this.point = point;
  this.param = param;

  this.enableMassClear = this.param.enableMassClear === false ? false : true;
}
// 继承 API 的 BMap.Overlay
MyMapOverlay.prototype = new BMap.Overlay();

// 实现初始化方法
MyMapOverlay.prototype.initialize = function() {
  var self = this;

  var markerDOM = this.buildMarker();

  this.markerDOM = markerDOM;

  return this.markerDOM;
};

// 实现绘制方法
MyMapOverlay.prototype.draw = function(){
  // 根据地理坐标转换为像素坐标，并设置给容器
  var position = this.map.pointToOverlayPixel(this.point);
  this.markerDOM.style.left = position.x - $(this.markerDOM).width() / 2 + "px";
  this.markerDOM.style.top = position.y - $(this.markerDOM).height() / 2 + "px";
};

MyMapOverlay.prototype.getPosition = function() {
  return this.point;
};

MyMapOverlay.prototype.buildMarker = function() {
  var self = this;
  var markerType = this.param.markerType;
  var markerClassName = this.param.markerClassName;
  var markerBackgroundPositionY = 0 - this.param.markerIndex * 30;
  var $markerResultPanel = $(this.param.markerResultPanel);
  var markerTitle = this.param.markerTitle;
  // 创建 div 元素，作为自定义覆盖物的容器
  var divHTML = '';
  if (markerType === 'poi') {
    divHTML = '<div class="'+ markerClassName +'" ';
    divHTML += 'title="'+ markerTitle +'" ';
    divHTML += 'style="background-position:0 '+ markerBackgroundPositionY +'px">';
    divHTML += '</div>';
  } else if (markerType === 'house') {

    divHTML = '<span class="address">'+ this.param.name +'</span>';
  }

  var $div = $(divHTML);
  var div = $div[0];

  // 将 div 添加到覆盖物容器中
  this.map.getPanes().markerPane.appendChild(div);

  if (markerType === 'poi') {
    $div.hover(function() {
      $div.css({
        'background-position': '-35px ' + markerBackgroundPositionY + 'px',
        'z-index': 2
      });

      var $resultItem = $markerResultPanel.find('dd').eq(self.param.markerIndex);
      $resultItem.addClass('hover').find('.result-marker-letter').css('background-position', '-97px ' + markerBackgroundPositionY + 'px');

      var resultItemOffsetTop = $resultItem[0].offsetTop;
      var resultPanelScrollTop = $markerResultPanel.scrollTop();
      var resultPanelHeight = $markerResultPanel.height();
      if (resultItemOffsetTop > resultPanelScrollTop + resultPanelHeight) {
        $markerResultPanel.scrollTop(resultItemOffsetTop - resultPanelHeight + $resultItem[0].clientHeight);
      } else if (resultItemOffsetTop < resultPanelScrollTop) {
        $markerResultPanel.scrollTop(resultItemOffsetTop);
      }

    }, function() {
      $div.css({
        'background-position': '0 ' + markerBackgroundPositionY + 'px',
        'z-index': 1
      });

      var $resultItem = $markerResultPanel.find('dd').eq(self.param.markerIndex);
      $resultItem.removeClass('hover').find('.result-marker-letter').css('background-position', '-69px ' + markerBackgroundPositionY + 'px');
    });
  }
  return div;
};
var MapApp = {
	mapContainer : document.getElementById('mapContainer'),
	mapResultPanel : document.getElementById('mapResultPanel'),
	mapTags : document.getElementById('mapTags'),
	overlayClass : MyMapOverlay,
	houseMarkerClassName : 'lp-map-mark',
	poiMarkerClassName : 'marker-letter',
	tags : {
		bus : '公交站',
		subway : '地铁站',
		education : '教育',
		hospital : '医院',
		bank : '银行',
		fun : '休闲娱乐',
		shopping : '购物',
		catering : '餐饮',
		sport : '运动健身'
	},

	// 检索'全部'时，拼装关键字
	getAllKw : function() {
		var kw = [], k, i, tags = this.tags, tag, _tag;
		for (k in tags) {
			tag = tags[k];
			if (Object.prototype.toString.call(tag) === '[object Array]') {
				for (i = 0; _tag = tag[i]; i++) {
					kw.push(_tag);
				}
			} else {
				kw.push(tag);
			}
		}
		return kw;
	},

	main : function() {
		this.param = $(this.mapContainer).data();
		this.point = new BMap.Point(this.param.lng, this.param.lat);

		this.initMap();
		this.initLocalSearch();
	},

	initMap : function() {
		this.map = new BMap.Map(this.mapContainer, {
			minZoom : 12
		});
		this.map.centerAndZoom(this.point, 14);
		this.map.enableKeyboard();
		this.map.addControl(new BMap.NavigationControl());
		this.map.addControl(new BMap.ScaleControl());
		this.map.enableScrollWheelZoom(true); 
		var houseMarker = new this.overlayClass(this.map, this.point, {
			markerType : 'house',
			markerClassName : this.houseMarkerClassName,
			name : this.param.name,
			enableMassClear : false
		});
		this.map.addOverlay(houseMarker);
	},

	initLocalSearch : function() {
		var self = this;

		var $mapTags = $(this.mapTags);
		var $mapResultPanel = $(this.mapResultPanel);
		var $markerPane = $(this.map.getPanes().markerPane);

		$mapResultPanel.css({
			position : 'relative'
		});

		// 各TAG的点击
		$mapTags.on('click', 'a', function() {
			var $this = $(this);
			var rel = $this.data('rel');
			var keyword;

			if (rel == 'all') {
				keyword = self.getAllKw();
			} else {
				keyword = self.tags[rel];
			}

			$this.addClass('tagActive').siblings().removeClass('tagActive');

			self.search(keyword);
		});

		// 右侧结果面板的hover响应到标注
		$mapResultPanel.on(
				'mouseenter',
				'dd',
				function() {
					var $self = $(this);
					var index = $mapResultPanel.find('dd').index($self);
					var backgroundPositionY = 0 - index * 30;
					$markerPane.find('.' + self.poiMarkerClassName).eq(index)
							.css(
									{
										'background-position' : '-35px '
												+ backgroundPositionY + 'px',
										'z-index' : 2
									});

					$self.addClass('hover').find('.result-marker-letter').css(
							'background-position',
							'-97px ' + backgroundPositionY + 'px');

				}).on(
				'mouseleave',
				'dd',
				function() {
					var $self = $(this);
					var index = $mapResultPanel.find('dd').index($self);
					var backgroundPositionY = 0 - index * 30;
					$markerPane.find('.' + self.poiMarkerClassName).eq(index)
							.css(
									{
										'background-position' : '0 '
												+ backgroundPositionY + 'px',
										'z-index' : 1
									});

					$self.removeClass('hover').find('.result-marker-letter')
							.css('background-position',
									'-69px ' + backgroundPositionY + 'px');

				});

		// 实例化周边搜索
		this.local = new BMap.LocalSearch(this.point, {
			renderOptions : {
				map : this.map,
				panel : this.mapResultPanel,
				selectFirstResult : false
			}
		});

		// 第一次先执行公交车站检索
		this.search('公交站');
	},

	search : function(kw) {
		var pageCapacity = 10;
		var radius = 1000;

		if (Object.prototype.toString.call(kw) === '[object Array]') {
			pageCapacity = 12 / kw.length;
		}
		this.map.setZoom(16);
		switch (kw) {
		case '医院':
		case '教育':
			radius = 5000;
			break;
		case '银行':
		case '休闲娱乐':
		case '购物':
		case '运动健身':
			radius = 3000;
			break;
		default:
			radius = 1000;
		}
		if (radius > 1000) {
			this.map.setZoom(14);
		}

		// 设置页的大小
		this.local.setPageCapacity(pageCapacity);

		// 得先清除掉地图上已经标注的Marker
		this.clearOverlays();

		$(this.mapResultPanel).html('<p style="padding:6px;">数据加载中...</p>');

		// 关键字半径1000米内搜索
		this.local.searchNearby(kw, this.point, radius);
	},

	/**
	 * 自定义的清除标注方法，过虑掉不允许清除的标注
	 */
	clearOverlays : function() {
		var markers = this.map.getOverlays();
		for (var i = 0, marker; marker = markers[i]; i++) {
			if (marker.enableMassClear === false) {
				continue;
			}
			this.map.removeOverlay(marker);
		}
	},

	/**
	 * 添加标注于地图
	 * 
	 * @param {LocalResult}
	 *            results 百度返回的结果对象，多关键字检索时为Array
	 */
	addOverlay : function(results) {
		var self = this;
		var marker, result, i = 0, j, poi, count = 0;

		while (result = results[i]) {
			for (j = 0; poi = result.getPoi(j); j++) {
				// 调用自定义的覆盖物类把POI标注创建出来
				marker = new self.overlayClass(self.map, poi.point, {
					markerType : 'poi',
					markerClassName : self.poiMarkerClassName,
					markerResultPanel : self.mapResultPanel,
					markerIndex : count,
					markerTitle : poi.title
				});
				// 添加到MAP
				self.map.addOverlay(marker);
				count++;
			}
			i++;
		}
	},

	/**
	 * 渲染右侧的结果面板
	 * 
	 * @param {LocalResult}
	 *            results 百度返回的结果对象，多关键字检索时为Array
	 */
	renderList : function(results) {
		// console.log(results.get(ov));
		var $mapResultPanel = $(this.mapResultPanel);
		var marker, result, i = 0, j, poi, count = 0;

		var listHTML = [], listItem = '';
		while (result = results[i]) {
			listItem += '<dl>';
			listItem += '<dt>' + result.keyword + '</dt>';

			if (result.getNumPois() === 0) {
				listItem += '<div class="no-surrounding">';
				listItem += '<div class="dt-title"></div>';
				listItem += '<i></i>';
				listItem += '<span>附近没有相关配套设施</span>';
				listItem += '</div>';
			}

			for (j = 0; poi = result.getPoi(j); j++) {
				listItem += '<dd>';
				listItem += '<i class="result-marker-letter" style="background-position: -69px -'
						+ count * 30 + 'px"></i>';
				listItem += '<p class="highlight-txt">';
				listItem += '<span class="name" title="' + poi.title + '">'
						+ poi.title + '</span>';
				listItem += '<span class="distance">'
						+ parseInt(this.map.getDistance(this.point, poi.point),
								10) + '米</span>';
				listItem += '</p>';
				listItem += '<p class="addr">' + poi.address + '</p>';
				listItem += '</dd>';

				count++;
			}
			listItem += '</dl>';

			i++;
		}
		listHTML.push(listItem);
		$mapResultPanel.html(listHTML.join(''));
	}
};
$(function(){
	MapApp.main();
})